/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

int softerror(int);

extern void log(const char *, ...);

void
serve(int fd)
{
	int nfd, pid;

	for (;;) {
		/*
		 * Block until a connect indication arrives;
		 * then accept it.
		 */
		nfd = accept(fd, NULL, NULL);
		if (nfd < 0) {
			log("cannot accept connection: %s",
			  strerror(errno));

			/*
			 * If this was a recoverable error,
			 * then try again.
			 */
			if (softerror(errno))
				continue;
			else
				exit(1);
		}

		/*
		 * Fork off a child process to handle the
		 * service request.
		 */
		if ((pid = fork()) < 0) {
			log("cannot create child: %s",
			  strerror(errno));
			if (!softerror(errno))
				exit(1);
		} else if (pid == 0) {
			/*
			 * The child doesn't need the file
			 * descriptor used to accept connections.
			 */
			close(fd);

			/*
			 * This is where the child would
			 * perform the service.
			 */
		}

		/*
		 * The parent doesn't need the file descriptor
		 * for the newly created connection.
		 */
		close(nfd);
	}
}

/*
 * Return 1 if error is recoverable and
 * 0 if unrecoverable.
 */
int
softerror(int num)
{
	switch (num) {
	case EAGAIN:
	case ENOSR:
	case ENODEV:
	case ENOSPC:
	case ENOMEM:
	case ENXIO:
	case ENOBUFS:
		return(1);

	default:
		return(0);
	}
}
